#!/usr/bin/perl -Tw
#
# LMS version 1.8.1 Shyla _ moded by ReYu
#
#  (C) 2001-2010 LMS Developers
#
#  Please, see the doc/AUTHORS for more information about authors!
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License Version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
#  USA.
#
#  $Id: lms-cashimport-Inteligo,v 1.1.2.2 2006/01/16 09:35:17 reyu Exp $

use strict;
use DBI;
use Config::IniFiles;
use Getopt::Long;
use HTTP::Cookies;
use HTTP::Request::Common qw(POST GET);
use Crypt::SSLeay;
use LWP::UserAgent;
use POSIX;
use POSIX qw(mktime strftime);
use HTML::TokeParser;
use Time::Local;
use vars qw($configfile $quiet $help $version);
use Digest::MD5;
use Text::Iconv;
use Digest::MD5 qw(md5_hex);

my $_version = '1.8.1 Shyla moded by Simson';

my %options = (
               "--config-file|C=s"     =>      \$configfile,
               "--quiet|q"             =>      \$quiet,
               "--help|h"              =>      \$help,
               "--version|v"           =>      \$version,
               );

Getopt::Long::config("no_ignore_case");
GetOptions(%options);

if($help)
{
       print STDERR <<EOF;
       lms-cashimport-inteligo, version $_version
               (C) 2001-2010 LMS Developers & ReYu & Simson

       -C, --config-file=/etc/lms/lms.ini      alternate config file (default: /etc/lms/lms.ini);
       -h, --help                      print this help and exit;
       -v, --version                   print version info and exit;
       -q, --quiet                     suppress any output, except errors;

EOF
       exit 0;
}

if($version)
{
       print STDERR <<EOF;
       lms-cashimport-inteligo, version $_version
               (C) 2001-2010 LMS Developers & ReYu & Simson

EOF
       exit 0;
}

if(!$configfile)
{
       $configfile = "/etc/lms/lms.ini";
}

if(!$quiet)
{
       print STDOUT "lms-cashimport-inteligo, version $_version\n";
       print STDOUT "(C) 2001-2010 LMS Developers & ReYu & Simson\n";
       print STDOUT "Using file $configfile as config.\n";
}

if(! -r $configfile)
{
       print STDERR "Fatal error: Unable to read configuration file $configfile, exiting.\n";
       exit 1;
}

my $ini = new Config::IniFiles -file => $configfile;
print @Config::IniFiles::errors;

my $dbtype = $ini->val('database', 'type') || 'mysql';
my $dbhost = $ini->val('database', 'host') || 'localhost';
my $dbuser = $ini->val('database', 'user') || 'root';
my $dbpasswd = $ini->val('database', 'password') || '';
my $dbname = $ini->val('database', 'database') || 'lms';

my $login_i = $ini->val('cashimport_inteligo', 'id') || '12345678';
my $passwd_i = $ini->val('cashimport_inteligo', 'pass') ||  'abcdef';
my $account_id = $ini->val('cashimport_inteligo', 'account') || '111111111111';
my $time_start = $ini->val('cashimport_inteligo', 'wyciag_start') || strftime "%Y-%m-%d", localtime(time-86400);
my $time_end = $ini->val('cashimport_inteligo', 'wyciag_end') || strftime "%Y-%m-%d", localtime(time);
my $beg_date_y = strftime "%Y", localtime(time-86400);
my $beg_date_m = strftime "%m", localtime(time-86400);
my $beg_date_d = strftime "%d", localtime(time-86400);
my $end_date_y = strftime "%Y", localtime(time);
my $end_date_m = strftime "%m", localtime(time);
my $end_date_d = strftime "%d", localtime(time);
my $trans_type = $ini->val('cashimport_inteligo', 'trans_type') || 'CREDIT';
my $file_out_dir = $ini->val('cashimport_inteligo', 'history_dir') || '/var/lms/cashimport';

my $dbase;

if($dbtype eq "mysql")
{
       $dbase = DBI->connect("DBI:mysql:database=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
elsif($dbtype eq "postgres")
{
       $dbase = DBI->connect("DBI:Pg:dbname=$dbname;host=$dbhost","$dbuser","$dbpasswd", { RaiseError => 1 });
}
else
{
       print STDERR "Fatal error: unsupported database type: $dbtype, exiting.\n";
       exit 1;
}

# Definicja Agenta http
my $sd ='';
my $cookie_jar = HTTP::Cookies->new;
my $ua = new LWP::UserAgent;
$ua->timeout(30);
$ua->agent('Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2');
$ua->env_proxy();

# Wchodzimy na stron
my $get = new HTTP::Request(GET => 'https://inteligo.pl/secure');
my $res = $ua->request($get);
if ($res->content =~ /Przepraszamy/ ){
       print "ERROR: System chwilowo niedostpny", "\n";
       exit 1;
}

if ($res->content =~ /Identyfikator/ ){
       my $p = HTML::TokeParser->new(\$res->content);
       while ( my $token1 = $p->get_tag("input")){
               if (my $ur = $token1->[1]{name} eq "sd"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $sd = $url2; }
       }
}

print "Logujemy do Serwisu ... \t";
my $post = POST 'https://inteligo.pl/secure', [
  form_name => "login",
  menu => "",
  button => "",
  "button.x" => "0",
  cif => "$login_i",
  password => "$passwd_i",
  token_password => "",
  "btn_ok.x" => "23",
  "btn_ok.y" => "14",
  skin => "Granat",
  sd => "$sd"
  ];
my $res1 = $ua->request($post);

my $ias_sid = '';
my $back_reload_protection = '';
my $ias_form_protect = '';
my $ias_navi_map = '';

if ($res1->content =~ /Stan Twoich/ ){
       print "Jestesmy zalogowani", "\n";
       my $p = HTML::TokeParser->new(\$res1->content);
       while ( my $token1 = $p->get_tag("input")){
#if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
#if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
               if (my $ur = $token1->[1]{name} eq "sd"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $sd = $url2; }
       }
} else {
       print "Exit\n";
       exit 1;
}

print "Przechodzimy do historii ...\t";
my $post1 = POST 'https://inteligo.pl/secure', [
  section => "mojerachunki",
  return_point => "1",
  form_name => "accounts",
  menu => "history",
  button => "",
  skin => "Granat",
  sd => "$sd"
  ];
$res = $ua->request($post1);
if ($res->content =~ /Historia rachunku/ ){
       print "Strona Historii Rachunku \n";
       my $p1 = HTML::TokeParser->new(\$res->content);
       while ( my $token1 = $p1->get_tag("input")){
#if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
#if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
               if (my $ur = $token1->[1]{name} eq "sd"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $sd = $url2; }
       }
} else {
       print "Exit\n";
       exit 1;
}
print "Zmieniamy filtr historii ...\t";
my $post2 = POST 'https://inteligo.pl/secure', [
  section => "mojerachunki",
  return_point => "1",
  form_name => "history",
  menu => "",
  selected_acc => "$account_id",
  beg_date_y => "$beg_date_y",
  beg_date_m => "$beg_date_m",
  beg_date_d => "$beg_date_d",
  end_date_y => "$end_date_y",
  end_date_m => "$end_date_m",
  end_date_d => "$end_date_d",
  search_text => "",
  search_type => "all_words",
  sel_tx_filter_type => "ALL",
  amount_min => "",
  amount_max => "",
  search_account => "",
  "btn_find.x" => "35",
  "btn_find.y" => "7",
  export_type => "PDF",
  button => "",
  skin => "Granat",
  sd => "$sd"
  ];
$res = $ua->request($post2);
if ($res->content =~ /Nie znaleziono transakcji/ ) {
       print "Brak transakcji\n";
       exit 1;
}
if ($res->content =~ /Historia rachunku/ ) {
       print "Zmieniono filtr $time_start - $time_end\n";
       my $p1 = HTML::TokeParser->new(\$res->content);
       while (my $token1 = $p1->get_tag("input")){
#if (my $ur = $token1->[1]{name} eq "ias_sid"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_sid = $url2; }
#if (my $ur = $token1->[1]{name} eq "back_reload_protection"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $back_reload_protection = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias.form_protect"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_form_protect = $url2; }
#if (my $ur = $token1->[1]{name} eq "ias_navi_map"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $ias_navi_map = $url2; }
               if (my $ur = $token1->[1]{name} eq "sd"){ my $url = $token1->[1]{name} || "-"; my $url2 = $token1->[1]{value} || "-"; $sd = $url2; }
       }
} else {
       print "Exit\n";
       exit 1;
}

print "Szukamy plik historii ...\t";
my $post3 = POST 'https://inteligo.pl/secure', [
  section => "mojerachunki",
  return_point => "1",
  form_name => "history",
  menu => "",
  selected_acc => "$account_id",
  beg_date_y => "$beg_date_y",
  beg_date_m => "$beg_date_m",
  beg_date_d => "$beg_date_d",
  end_date_y => "$end_date_y",
  end_date_m => "$end_date_m",
  end_date_d => "$end_date_d",
  search_text => "",
  search_type => "all_words",
  sel_tx_filter_type => "ALL",
  amount_min => "",
  amount_max => "",
  search_account => "",
  export_type => "CSV",
  button => "export_list",
  skin => "Granat",
  sd => "$sd"
  ];
$res = $ua->request($post3);
if ($res->code == 302 ){
       print "Plik znaleziony\n";
} else {
#print "\n".$post3->as_string."\n";
       print "\n".$res->as_string."\n";
       print "Exit\n";
       exit 1;
}

print "Pobieramy plik historii ...\t";
my $post4 = GET 'https://inteligo.pl/'.$res->header("Location");
$res = $ua->request($post4);
if ($res->code == 200 ){
       print "Plik pobrany\n";
} else {
#print "\n".$post3->as_string."\n";
       print "\n".$res->as_string."\n";
       print "Exit\n";
       exit 1;
}
my @dana = split("\n", $res->content);

my $okCounter = 0;

for (my $x=1;$x<scalar(@dana);$x++){
#my $converter = Text::Iconv->new("ISO-8859-2" "UTF-8");
       my $line = $dana[$x];

   print "\n$line\n";
#$line = '2011-11-18,837,Przelew na rachunek,NAZWISKO IMIE ADRES Nr rach.: 11222233334444555566667777 Tytu: LMS999,"+56,78","1 234,56"';
#$line =~ m/(.*),(.*),Przelew na rachunek,"?([^"]*) Nr rach.: ([^"]*) Tytu.: ([^"]*)"?,"(.*)","(.*)"/;

#$line = '"841","2011-11-30","2011-11-30","Przelew na rachunek","+56.78","PLN","1234.56","11222233334444555566667777","NAZWISKO IMIE ADRES","NAZWISKO IMIE ADRES","Nr rach.: 11222233334444555566667777","Tytu: LMS999"';
       $line =~ m/"(.*)","(.*)",".*","Przelew na rachunek","(.*)","PLN",".*","(.*)","(.*)",".*",".*","Tytu.: (.*)"/;
       if (!$1) {
               next;
       }

       my $statementid = $1;
       my $date = $2;
       my $value = $3;
       my $account = $4;
       my $sender = $5;
       my $description = $6;
       my $customerid = 0;
       my $customer = "BRAK!";
   $value =~ s/,/./g;
       my $fullDescription = $sender."<BR/>\n".$description."<BR/>\n".$account."<BR/>\n";

       $description=~ m/LMS ?([0-9][0-9][0-9])/;
       my $lmsString="$1";

       if($account) {
               my $sdbq;
               if ($lmsString) {
                       $sdbq = $dbase->prepare("SELECT id, lastname, name, address FROM customers WHERE notes LIKE '%LMS$lmsString%'");
               } else {
                       $sdbq = $dbase->prepare("SELECT id, lastname, name, address FROM customers WHERE notes LIKE '%$account%'");
               }
               $sdbq->execute;
               if (my $row = $sdbq->fetchrow_hashref()){
                       $customerid = $row->{'id'};
                       $customer = $row->{'lastname'}." ".$row->{'name'}." ".$row->{'address'};
               }
       }

       my $hash = "BankStatementId:".$statementid;
       my $sdbq = $dbase->prepare("SELECT id FROM cashimport WHERE hash = '$hash'");
       $sdbq->execute();
       my $row = $sdbq->fetchrow_hashref();
       if ($row->{'id'}) {
               print "ERROR: Hash exists, ignoring $customer $sender", "\n";
       } elsif ($customerid == '0') {
               #print ("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', '$customerid', '$hash');");
       print "Data: $date\nWysa: $sender\nAutomatyczna identyfikacja klienta: $customer ($customerid)\nNr konta:$account\nWarto: $value\nOpis: $description\nHash: $hash\n";
               $sdbq = $dbase->prepare("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', NULL, '$hash');");
               $sdbq->execute();
               $okCounter ++;
       } else {
       #print "\nINSERT INTO cash ( time , userid , type , value , customerid , comment ) VALUES (UNIX_TIMESTAMP('$date'), '2', '1', '$value', '$customerid', '$fullDescription' );\n";
       $sdbq = $dbase->prepare("INSERT INTO cash ( time , userid , type , value , customerid , comment ) VALUES (UNIX_TIMESTAMP('$date'), '2', '1', '$value', '$customerid', '$fullDescription' );");
       $sdbq->execute();

       $sdbq = $dbase->prepare("INSERT INTO cashimport ( Date , Value , Customer , Description , CustomerId , Hash , Closed ) VALUES (UNIX_TIMESTAMP('$date'), '$value', '$customer', '$fullDescription', '$customerid', '$hash', '1');");
       $sdbq->execute();
       print "Data: $date\nWysa: $sender\nAutomatyczna identyfikacja klienta: $customer ($customerid)\nNr konta:$account\nWarto: $value\nOpis: $description\nHash: $hash\n";
               print "\nCustomerID znaleziony! Wpata zaksigowana automatycznie!\n";
       $okCounter ++;
       }
}

print "OK: $okCounter\n";
if ($okCounter eq 0) {
       exit 2;
}
exit 0;

print "ERROR: Nieznany bd";
exit 1;

